// (C) 2006 Witold Bolt; ja@hope.art.pl

// ten program jest stanowczo za dugi ;)

#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

typedef double vec4[4];

// Funkcja wylicza praw stron rwnania dla danego t
// oraz wektora x. Wynik zwracany jest przez argument result.
void f (double t, vec4 x, vec4 result) {
    result[0] = x[1];
    result[1] = x[2];
    result[2] = x[3];
    result[3] = -1.0 * (exp(1+t)) * x[0] - t * x[1] - (t+1) * x[2] +
                -1.0*(1+sqrt(1+t*t)) * x[3] + (1+t*t);
}

// Funkcja pomocnicza wykorzystywana przy wyliczaniu kolejnych
// elementw cigu -- realizuje dodawanie wektorw przy czym wektor
// y jest przed dodaniem mnoony przez h.
void add (vec4 x, double h, vec4 y) {
    x[0] += h*y[0];
    x[1] += h*y[1];
    x[2] += h*y[2];
    x[3] += h*y[3];
}

// prosta funkcja kopiuje wartosci z jednego wektora do drugiego ...
void copy (vec4 src, vec4 dest) {
    dest[0] = src[0];
    dest[1] = src[1];
    dest[2] = src[2];
    dest[3] = src[3];
}

// funkcja generuje kolejne punkty metody Eulera
// a - punkt pocztkowy
// x_0 - warunek pocztkowy
// h - skok :)
// plot - plik wynikowy
// name - nazwa zmiennej w pliku do ktrej zapisa wyniki
void get_euler_points(double a, vec4 x_0, double h, FILE *plot, const char name[32]) {
    double t=a;
    vec4 x;
    vec4 result;

    copy(x_0,x);

    fprintf(plot,"pair[] %s = {", name);

    while(t<=2.0) {
        fprintf(plot,"(%f,%f),",t,x[0]);
        f(t,x,result);
        add(x,h,result);
        t+=h;
    }

    fflush(plot);
    fseek(plot,-1,SEEK_CUR);
    fprintf(plot,"};\n");

}


// funkcja generuje kolejne punkty metody Runge--Kuttego
// a - punkt pocztkowy
// x_0 - warunek pocztkowy
// h - skok :)
// plot - plik wynikowy
// name - nazwa zmiennej w pliku do ktrej zapisa wyniki
void get_rk_points(double a, vec4 x_0, double h, FILE *plot, const char name[32]) {
    double t=a;
    vec4 x;
    copy(x_0,x);
    vec4 x2;
    vec4 k1,k2;

    fprintf(plot,"pair[] %s = {", name);

    while(t<=2.0) {
        fprintf(plot,"(%f,%f),",t,x[0]);
        copy(x,x2);

        f(t,x,k1);
        add(x,0.5*h,k1);

        f(t+h*0.5,x,k2);

        add(x2,h,k2);
        copy(x2,x);
        t+=h;
    }

    fflush(plot);
    fseek(plot,-1,SEEK_CUR);
    fprintf(plot,"};\n");
}

// gwna funkcja -- generuje wyjcie i takie tam nudy
int main(int argc, char **argv) {
    // liczba punktw do wyznaczenia
    int n;
    char filename[256] = "plot.asy\0";

    if(argc<2) {
        printf("n = ");
        scanf("%d",&n);
    }

    else {
        n = atoi(argv[1]);
    }

    if(argc>=3) {
        strncpy(filename,argv[2],strlen(argv[2])+1);
    }
    // dugo maego odcinka
    double h = 2.0/(double)n;

    // punkt startowy
    double a = 0.0;

    // x zawiera warunek pocztkowy
    vec4 x = {0,0,0,0};

    printf("Generuje plik: %s\n",filename);

    FILE *plot = fopen(filename,"w");

    fprintf(plot,"import graph;\n");

    get_euler_points(a, x, h, plot, "euler_data");
    get_rk_points(a, x, h, plot, "rk_data");

    fprintf(plot,"size(6cm,7cm,IgnoreAspect);\n");

    fprintf(plot,"draw(graph(euler_data),green,\"{\\tiny Euler}\",MarkFill[0]);\n");
    fprintf(plot,"draw(graph(rk_data),blue,\"{\\tiny Runge--Kutty}\",MarkFill[0]);\n");

    fprintf(plot,"xaxis(\"$x$\",fontsize(6),LeftTicks,EndArrow);\n");
    fprintf(plot,"yaxis(\"$y$\",0,fontsize(6),RightTicks,EndArrow);\n");
    fprintf(plot,"attach(point(N),legend(S,0.2,0.2,fontsize(6)));\n");
    return(0);
}
